
%(Runge-Kutta [N = 4]) Routine - Here the general procedure for calcuating 
%                                the fourth order Runge-Kutta approximation
%                                to a first order derivative is used.
%                            
%         
%Detailed explanation - In the case of the predator-prey model, initial
%                       conditions for x(0) and y(0) are used to 
%                       iteratively solve the system of differential 
%                       equations via the RK4 routine.
%                           
%               
%INPUT      - x0 is the initial number of prey. i.e., x(0)
%           - y0 is the initial number of predators. i.e., y(0)
%           - t0 is starting time interval
%           - h is the step size
%           - n is the number of intervals
%            
%OUTPUT     - R is the vector containg the solution curve ordinates, 
%             abscissas, and their respective axes
%
%%
function [R] = RK4(x0,y0,t0,h,n)

%Pre-allocate space for x,y,t variables
x = zeros(1, n+1);
y = zeros(1, n+1);
t = zeros(1, n+1);

%Initial conditions
x(1) = x0; 
y(1) = y0;
t(1) = t0;

%Change of notation (convenience)
m = 0.5*h;

%Iterating over all n intervals
for k = 1:n
    %f_xy,m and g_xy.m are the differential equations for the prey and 
    %predator respectively. (f_1,g_1) through (f_4,g_4) are function
    %evaluations at whole number time steps, and fractional number time
    %steps
    f_1 = f_xy(x(k), y(k));
    g_1 = g_xy(x(k), y(k));
    
    f_2 = f_xy(x(k) + m*f_1, y(k) + m*g_1);
    g_2 = g_xy(x(k) + m*f_1, y(k) + m*g_1);
      
    f_3 = f_xy(x(k) + m*f_2, y(k) + m*g_2);
    g_3 = g_xy(x(k) + m*f_2, y(k) + m*g_2);
      
    f_4 = f_xy(x(k) + h*f_3, y(k) + h*g_3);
    g_4 = g_xy(x(k) + h*f_3, y(k) + h*g_3);

    %The solutions x(t) and y(t) are found via iteration 
    x(k+1) = x(k) + h*(f_1 + 2*f_2 + 2*f_3 + f_4)/6;
    y(k+1) = y(k) + h*(g_1 + 2*g_2 + 2*g_3 + g_4)/6; 
    
    %Increment time
    t(k+1) = t(k) + h; 
end
%R is the solution vector containing required numerical data
R = [t' x' y'];
end

